@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
-__all__ = ["BaseClassifier"]
+__all__ = [
+    "BaseClassifier",
+    "classifier_list",
+]
 __author__ = ["Markus Löning"]
 
 import numpy as np
@@ -7,6 +10,32 @@ import numpy as np
 from sktime.base import BaseEstimator
 from sktime.utils.validation.panel import check_X
 
+"""
+Main list of classifiers extending this class. For clarity, some utility classifiers,
+such as Proximity Stump, are not listed.
+"""
+classifier_list = [
+    # in classification/distance_based
+    "ProximityForest",
+    # "KNeighborsTimeSeriesClassifier",
+    # "ElasticEnsemble",
+    # "ShapeDTW",
+    # in classification/dictionary_based
+    "BOSS",
+    "ContractableBOSS",
+    "TemporalDictionaryEnsemble",
+    "WEASEL",
+    "MUSE",
+    # in classification/interval_based
+    "RandomIntervalSpectralForest",
+    "TimeSeriesForest",
+    "CanonicalIntervalForest",
+    # in classification/shapelet_based
+    "ShapeletTransformClassifier",
+    "ROCKET",
+    "MrSEQLClassifier",
+]
+
 
 class BaseClassifier(BaseEstimator):
     """
