@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 __all__ = [
-    "TimeSeriesForest",
+    "TimeSeriesForestClassifier",
     "RandomIntervalSpectralForest",
     "SupervisedTimeSeriesForest",
 ]
 
 from sktime.classification.interval_based._rise import RandomIntervalSpectralForest
 from sktime.classification.interval_based._stsf import SupervisedTimeSeriesForest
-from sktime.classification.interval_based._tsf import TimeSeriesForest
+from sktime.classification.interval_based._tsf import TimeSeriesForestClassifier
