@@ -13,11 +13,13 @@ __all__ = [
     "ForecastingRandomizedSearchCV",
     "SlidingWindowSplitter",
     "SingleWindowSplitter",
+    "ExpandingWindowSplitter",
     "temporal_train_test_split",
     "NaiveForecaster",
     "ExponentialSmoothing",
     "ThetaForecaster",
     "AutoARIMA",
+    "ARIMA",
     "PolynomialTrendForecaster",
     "TransformedTargetForecaster",
     "Deseasonalizer",
@@ -34,6 +36,7 @@ __all__ = [
     "NormalHedgeEnsemble",
     "NNLSEnsemble",
     "OnlineEnsembleForecaster",
+    "evaluate",
 ]
 
 import numpy as np
@@ -44,20 +47,30 @@ from sktime.datasets import load_longley
 from sktime.datasets import load_lynx
 from sktime.datasets import load_shampoo_sales
 from sktime.forecasting.arima import AutoARIMA
+from sktime.forecasting.arima import ARIMA
 from sktime.forecasting.base import ForecastingHorizon
 from sktime.forecasting.compose import EnsembleForecaster
 from sktime.forecasting.compose import ReducedForecaster
 from sktime.forecasting.compose import TransformedTargetForecaster
 from sktime.forecasting.exp_smoothing import ExponentialSmoothing
 from sktime.forecasting.model_selection import CutoffSplitter
-from sktime.forecasting.model_selection import (
-    ForecastingGridSearchCV,
-    ForecastingRandomizedSearchCV,
-)
+from sktime.forecasting.model_selection import ExpandingWindowSplitter
+from sktime.forecasting.model_selection import ForecastingGridSearchCV
+from sktime.forecasting.model_selection import ForecastingRandomizedSearchCV
 from sktime.forecasting.model_selection import SingleWindowSplitter
 from sktime.forecasting.model_selection import SlidingWindowSplitter
 from sktime.forecasting.model_selection import temporal_train_test_split
+from sktime.forecasting.model_evaluation import evaluate
 from sktime.forecasting.naive import NaiveForecaster
+from sktime.forecasting.online_learning._online_ensemble import (
+    OnlineEnsembleForecaster,
+)
+from sktime.forecasting.online_learning._prediction_weighted_ensembler import (
+    NNLSEnsemble,
+)
+from sktime.forecasting.online_learning._prediction_weighted_ensembler import (
+    NormalHedgeEnsemble,
+)
 from sktime.forecasting.theta import ThetaForecaster
 from sktime.forecasting.trend import PolynomialTrendForecaster
 from sktime.performance_metrics.forecasting import MASE
@@ -67,10 +80,3 @@ from sktime.performance_metrics.forecasting import smape_loss
 from sktime.transformations.series.detrend import Deseasonalizer
 from sktime.transformations.series.detrend import Detrender
 from sktime.utils.plotting import plot_series
-from sktime.forecasting.online_learning._prediction_weighted_ensembler import (
-    NormalHedgeEnsemble,
-    NNLSEnsemble,
-)
-from sktime.forecasting.online_learning._online_ensemble import (
-    OnlineEnsembleForecaster,
-)
