@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # copyright: sktime developers, BSD-3-Clause License (see LICENSE file)
 
-__author__ = ["Markus Löning", "Tomasz Chodakowski"]
+__author__ = ["Markus Löning", "Tomasz Chodakowski", "Martin Walter"]
 __all__ = [
     "MASE",
     "sMAPE",
